<?php

/**
 * This file contains QUI\Health\Linkchecker\Job
 */
namespace QUI\Health\Linkchecker;

use QUI;
use QUI\Health\Linkchecker\Utils\Url;

/**
 * Class Jobs
 */
class Job extends QUI\CRUD\Child
{
    /**
     * @var Handler
     */
    protected $Handler;

    /**
     * @var Url
     */
    protected $Url;

    /**
     * @var array
     */
    protected $checked = array();

    /**
     * Job constructor.
     *
     * @param int $id
     * @param Handler $Factory
     */
    public function __construct($id, Handler $Factory)
    {
        parent::__construct($id, $Factory);

        $this->Handler = new Handler();
        $this->Url     = new Url();
    }

    /**
     * Return all checked sites
     *
     * @return array
     */
    public function getData()
    {
        return QUI::getDataBase()->fetch(array(
            'from' => $this->Handler->getDataBaseLinkcheckerTableName(),
            'where' => array(
                'jobId' => $this->getId()
            )
        ));
    }

    /**
     * Execute the linkchecking job
     */
    public function execute()
    {
        if ($this->getAttribute('status') == 1) {
            return;
        }

        $projectName = $this->getAttribute('project');
        $projectLang = $this->getAttribute('lang');

        $Project = QUI::getProject($projectName, $projectLang);

        if ($this->getAttribute('lastrun') == '0000-00-00 00:00:00') {
            $ids     = array();
            $siteIds = $Project->getSitesIds();

            foreach ($siteIds as $data) {
                $ids[] = $data['id'];
            }

            $this->setAttribute('siteIds', implode($ids, ','));

            QUI::getDataBase()->update(
                $this->Handler->getDataBaseTableName(),
                array(
                    'siteids' => $this->getAttribute('siteIds'),
                    'lastrun' => date('Y-m-d H:i:s')
                ),
                array('id' => $this->getId())
            );
        }

        $allIds = explode(',', $this->getAttribute('siteids'));
        $done   = array();

        if ($this->getAttribute('done')) {
            $done = explode(',', $this->getAttribute('done'));
        }

        $diff = array_diff($allIds, $done);

        foreach ($diff as $siteId) {
            $processed = $this->processingSiteId($siteId);

            if ($processed === true) {
                $done[] = $siteId;

                QUI::getDataBase()->update(
                    $this->Handler->getDataBaseTableName(),
                    array('done' => implode(',', $done)),
                    array('id' => $this->getId())
                );
            }
        }

        QUI::getDataBase()->update(
            $this->Handler->getDataBaseTableName(),
            array('status' => 1),
            array('id' => $this->getId())
        );
    }

    /**
     * Processing one site
     *
     * @param integer $siteId
     * @return boolean
     */
    protected function processingSiteId($siteId)
    {
        $projectName = $this->getAttribute('project');
        $projectLang = $this->getAttribute('lang');

        $Project = QUI::getProject($projectName, $projectLang);
        $host    = $Project->getVHost(true, true);

        try {
            $Site = $Project->get($siteId);

        } catch (QUI\Exception $Exception) {
            QUI::getDataBase()->insert(
                $this->Handler->getDataBaseLinkcheckerTableName(),
                array(
                    'jobId' => $this->getId(),
                    'project' => $Project->getName(),
                    'lang' => $Project->getLang(),
                    'siteid' => $siteId,
                    'date' => date('Y-m-d H:i:s'),
                    'url' => '',
                    'errors' => '',
                    'linksChecked' => 0,
                    'data' => '',
                    'headercode' => 404
                )
            );

            return true;
        }

        $url    = $host . $Site->getUrlRewritten();
        $result = $this->Url->checkUrl($url);

        if ($result !== 200) {
            QUI::getDataBase()->insert(
                $this->Handler->getDataBaseLinkcheckerTableName(),
                array(
                    'jobId' => $this->getId(),
                    'project' => $Project->getName(),
                    'lang' => $Project->getLang(),
                    'siteid' => $siteId,
                    'date' => date('Y-m-d H:i:s'),
                    'url' => $url,
                    'errors' => '',
                    'linksChecked' => 0,
                    'data' => '',
                    'headercode' => $result
                )
            );

            return true;
        }

        // test all links to the site

        $errors = array();
        $links  = $this->Url->getLinksFromUrl($url);

        foreach ($links as $linkData) {
            if ($linkData['empty']) {
                $errors[] = $linkData;
            }

            $checkResult = $this->checkUrl($linkData);

            if ($checkResult === 200) {
                continue;
            }

            if ($checkResult === 303) {
                continue;
            }

            $errors[] = $checkResult;
        }

        QUI::getDataBase()->insert(
            $this->Handler->getDataBaseLinkcheckerTableName(),
            array(
                'jobId' => $this->getId(),
                'project' => $Project->getName(),
                'lang' => $Project->getLang(),
                'siteid' => $siteId,
                'date' => date('Y-m-d H:i:s'),
                'url' => $url,
                'errors' => count($errors),
                'linksChecked' => count($links),
                'data' => json_encode($errors),
                'headercode' => $result
            )
        );

        return true;
    }

    /**
     * Internal url check with cache
     *
     * @param array $linkData - link data array, data from Url->getLinksFromUrl()
     * @return array|integer
     */
    protected function checkUrl($linkData)
    {
        if (isset($this->checked[$linkData['url']])) {
            return $this->checked[$linkData['url']];
        }

        $headerCode = $this->Url->checkUrl($linkData['url']);

        if ($headerCode === 200) {
            return 200;
        }

        if ($headerCode === 303) {
            return 303;
        }

        $linkData['headerCode']          = $headerCode;
        $this->checked[$linkData['url']] = $linkData;

        return $this->checked[$linkData['url']];
    }
}
